@@ -317,10 +317,7 @@ class Swift(object):
         for header in env:
             if header.startswith('HTTP_') and env[header]:
                 key = header[5:]
-                if isinstance(env[header], str):
-                    headers[key] = str(env[header])
-                else:
-                    headers[key] = str(env[header])
+                headers[key] = str(env[header])
 
         try:
             container = obj = None
@@ -358,9 +355,13 @@ class Swift(object):
                     header.upper())
 
         # build object store details
+        if self.reseller_prefix:
+            resource_id = account.partition(self.reseller_prefix)[2] or path
+        else:
+            resource_id = account
         target = cadf_resource.Resource(
             typeURI='service/storage/object',
-            id=account.partition(self.reseller_prefix)[2] or path)
+            id=resource_id)
         target.metadata = resource_metadata
         target.action = method.lower()
 
